# Prepare workspace ####

# Define parent directory
parent.dir <- "C:/Final Report"

# Reset workspace 
rm(list = setdiff(ls(), "parent.dir"))
gc()

# Strings as characters
options(stringsAsFactors = FALSE)

# Child directories
source(file.path(parent.dir, "Shared Source Code",
                 "Define Child Directories.R"), echo = TRUE)

# Packages
source(file.path(parent.dir, "Shared Source Code",
                 "Install and Load Packages.R"), echo = TRUE)

# Custom functions
source(file.path(parent.dir, "Shared Source Code", "Define Functions.R"),
       echo = TRUE)



# Load Prepared Data ####

# Grants
grants.rda <- file.path(parent.dir, "Prepared Data", "Grants.rda")
load(grants.rda)
head(grants)

# Budgets
budget.rda <- file.path(parent.dir, "Prepared Data", "Budgets.rda")
load(budget.rda)
head(budget)

# PCC Sites
sites.rda <- file.path(parent.dir, "Prepared Data", "PCC Sites.rda")
load(sites.rda)
head(pcc_sites)

# PCC Equipment
equipment.rda <- file.path(parent.dir, "Prepared Data", "PCC Equipment.rda")
load(equipment.rda)
head(inst_upgr)




# Table 4. PCCs and Lab Hours ####

# Number of new and improved locations
locs <- data.table(dcast(
  pcc_sites, Award_Num ~ PCC.Type, value.var = "PCC.Type", 
  fun.aggregate = length), key = "Award_Num")
locs[, "Total PCCs" := Improved + New]

# Hours open per week
hours <- pcc_sites[, lapply(.SD, sum, na.rm = TRUE), keyby = Award_Num, 
                   .SDcols = c("week_hours", "weekend_hours")]
hours[, "Total weekly hours" := week_hours + weekend_hours]

# PCC Equipment
# Used to estimate "progress" - percentage of hours
prog <- inst_upgr[inst_upgr[quarter == "2012-12-31", list(
  wstn = WorkstatnInstall_Num, upgr = ConnectUpgrade_Num, 
  est = ConnectEstablish_Num), keyby = Award_Num]]
prog[, c("wstn", "upgr", "est") := Map("/", list(
  WorkstatnInstall_Num, ConnectUpgrade_Num, ConnectEstablish_Num),
  list(wstn, upgr, est))]
prog[, c("wstn", "upgr", "est") := Map(replace, .SD, lapply(.SD, ">", 1), 1),
     .SDcols = c("wstn", "upgr", "est")]
prog[, c("wstn", "upgr", "est") := 
       Map(replace, .SD, lapply(.SD, is.nan), NA_real_), 
     .SDcols = c("wstn", "upgr", "est")]
prog <- prog[!(rolled), list(Award_Num, quarter, pct = apply(
  do.call("cbind", list(wstn, upgr, est)), 1, mean, na.rm = TRUE))]

# Lookup progress and hours
d <- prog[hours[grants[Grant_Type == "PCC", Award_Num]], allow.cartesian = TRUE]

# Drop defunded
d <- d[!grants[Defunded != ""]]

# Replace missing
sdc <- c("week_hours", "weekend_hours", "Total weekly hours")
d[, sdc := Map(replace, .SD, lapply(.SD, is.na), 0), .SDcols = sdc, 
  with = FALSE]

# Multiply progress by hours...
sdc <- c("week_hours", "weekend_hours", "Total weekly hours")
d[, sdc := lapply(.SD, "*", d[, pct]), .SDcols = sdc, with = FALSE]
# ...multiply weekly hours by 13 to get quarterly totals...
d[, sdc := lapply(.SD, "*", 13), .SDcols = sdc, with = FALSE]
# ... and aggregate
d <- d[, lapply(.SD, sum), .SDcols = sdc, keyby = Award_Num]

# Lookup budget and locations
d <- d[budget[locs[grants[Grant_Type == "PCC"]]]]

# Drop defunded
d <- d[Defunded == ""]

# Replace missing
sdc <- c("Improved", "New", "Total PCCs")
d[, sdc := Map(replace, .SD, lapply(.SD, is.na), 0), .SDcols = sdc, 
  with = FALSE]

# Aggregate
sdc <- c("Improved", "New", "Total PCCs", "week_hours", "weekend_hours", 
         "Total weekly hours", "total")
cs <- d[, lapply(.SD, sum), .SDcols = sdc, by = Case_Study]
ab <- d[, lapply(.SD, sum), .SDcols = sdc,
        by = list(Case_Study = rep("All", nrow(d)))]

# Bind
d <- rbind(cs, ab)

# Calculate
d[, dollarper_pcc := total / `Total PCCs`]
d[, dollarper_hour := total / `Total weekly hours`]

# Millions
d[, total := total / 1e6]

# Reshape
d[, 2:ncol(d) := lapply(.SD, as.numeric), .SDcols = 2:ncol(d)]
d <- melt(d, id.vars = 1)

# Order and label variable (row name in final table)
d[, variable := factor(variable, c(
  "total", "New", "Improved", "Total PCCs", "dollarper_pcc", "week_hours",
  "weekend_hours", "Total weekly hours", "dollarper_hour"), c(
    "Total budget (millions)", "New PCCs established", "Existing PCCs improved",
    "Total PCCs", "Budget per PCC", "Total weekday lab hours (est.)", 
    "Total weekend lab hours (est.) ", "Total lab hours (est.)",
    "Budget per lab hour (est.)"))]

# Format numeric values
d[grep("Total budget", variable), v := sprintf("$%.0f", value)]
d[grep("Budgeted", variable), v := paste0(
  "$", formatC(value, digits = 2, format = "f", big.mark = ","))]
d[grep("Budgeted.*PCC$", variable), v := paste0(
  "$", formatC(value, digits = 0, format = "f", big.mark = ","))]
d[is.na(v), v := formatC(value, digits = 0, big.mark = ",", format = "f")]

# Table
pccs_hours_budg <- tabular(
  Heading("Activity") * variable ~ 
    Heading() * Case_Study * Heading() * v * Heading() * identity, d)
(pccs_hours_budg <- pccs_hours_budg[, -2])

# Write
write.xlsx.tabular(pccs_hours_budg, file.path(sec1, "Table 4.xlsx"))
