# Prepare workspace ####

# Define parent directory
parent.dir <- "C:/Final Report"

# Reset workspace 
rm(list = setdiff(ls(), "parent.dir"))
gc()

# Strings as characters
options(stringsAsFactors = FALSE)

# Child directories
source(file.path(parent.dir, "Shared Source Code", 
                 "Define Child Directories.R"), echo = TRUE)

# Packages
source(file.path(parent.dir, "Shared Source Code", 
                 "Install and Load Packages.R"), echo = TRUE)

# Custom functions
source(file.path(parent.dir, "Shared Source Code", "Define Functions.R"), 
       echo = TRUE)



# Load Prepared Data ####

# Grants
grants.rda <- file.path(parent.dir, "Prepared Data", "Grants.rda")
load(grants.rda)
head(grants)

# CCI Progress
progress.rda <- file.path(parent.dir, "Prepared Data", "CCI Progress.rda")
load(progress.rda)
head(cai_totals)

# CCI CAIs by Category
breakdown.rda <- file.path(parent.dir, "Prepared Data",
                           "CCI CAIs by Category.rda")
load(breakdown.rda)
head(connected_by_category)

# CCI Speed and Pricing
prspd.rda <- file.path(parent.dir, "Prepared Data", 
                       "CCI Speed and Pricing.rda")
load(prspd.rda)
head(price_speed)




# Table. Observed Subscription Speed and Pricing Changes from Before BTOP to 
# After BTOP ####

# Estimated in each category in evaluation study sample
## Total
ess <- cai_totals[grants][Grant_Type == "CCI"][
    Case_Study == "Evaluation Study Sample"][
      , sum(ComInst_Total_Num)]
## Breakdown
breakdown <- connected_by_category[grants, allow.cartesian = TRUE][
  Grant_Type == "CCI"][Case_Study == "Evaluation Study Sample"][
    , list(connected = sum(connected)), keyby = category]
breakdown[, connected := connected / sum(connected) * ess]

# Speeds and pricing
d <- copy(price_speed)

# Only summarize those with before and after values
d[is.na(price_before), price_after := NA_real_]
d[is.na(price_after), price_before := NA_real_]
d[is.na(price_mbps_before), price_mbps_after := NA_real_]
d[is.na(price_mbps_after), price_mbps_before := NA_real_]
d[is.na(speed_after), speed_before := NA_real_]
d[is.na(speed_before), speed_after := NA_real_]
 
# Reshape
d <- melt(d, id.vars = c("anchorname", "category"), measure.vars = c(
  "speed_before", "price_before", "price_mbps_before", "speed_after", 
  "price_after", "price_mbps_after"))[!is.na(value)]
d[, variable := as.character(variable)]
s <- strsplit(d[, variable], "_")
d[, btop := unlist(Map("[", s, lapply(s, length)))]
d[, variable := sapply(Map("[", s, lapply(lapply(s, length), "*", -1)), paste, 
                       collapse = "_")]

# Order factors
d[, variable := factor(variable, c("speed", "price", "price_mbps"))]
d[, btop := factor(btop, c("before", "after"))]

# Summarize
d <- d[, list(.N, med = median(value)), keyby = list(
  category, var1 = variable, btop)]
d <- melt(d, measure.vars = c("N", "med"), variable.name = "var2")
d <- Dcast(d, category ~ var1 + var2 + btop)
d[, pct_change := (price_mbps_med_before - price_mbps_med_after) / 
    price_mbps_med_before]

# Merge with breakdown
setkey(d, category)
d <- breakdown[d]

# Format values
d[, c("speed_N_after", "price_N_after", "price_mbps_N_before",
      "price_mbps_N_after") := NULL]
n <- names(d)
d[, connected := comma(round(connected, 0))]
d[, grep("N", n) := lapply(.SD, as.character), .SDcols = grep("N", n)]
d[, grep("med", n) := lapply(lapply(.SD, round, 0), comma), 
  .SDcols = grep("med", n)]
d[, pct_change := percent(round(pct_change, 2))]

# Header rows
headers <- matrix("", nrow = 3, ncol = 11)
headers[, 1] <- "Institution Type"
headers[, 2] <- "Connected in Evaluation Study Sample"
headers[1, 3:5] <- "Collected Speeds"
headers[1, 6:11] <- "Collected Prices"
headers[2:3, c(3, 6)] <- "N"
headers[2, 4:5] <- "Median Speed (Mbps)"
headers[2, 7:8] <- "Median Total Monthly Cost"
headers[2, 9:11] <- "Median Price per Mbps per Month"
headers[3, c(4, 7, 9)] <- "Before"
headers[3, c(5, 8, 10)] <- "After"
headers[3, 11] <- "Pct. Change"
headers <- as.data.table(headers)
setnames(headers, names(headers), names(d))
prsp_changes <- rbind(headers, d)

# Print to screen
prsp_changes

# Write
write.xlsx(prsp_changes, file.path(sec4, "Table 13.xlsx"), row.names = FALSE,
           col.names = FALSE)
