# Prepare workspace ####

# Define parent directory
parent.dir <- "C:/Final Report"

# Reset workspace 
rm(list = setdiff(ls(), "parent.dir"))
gc()

# Strings as characters
options(stringsAsFactors = FALSE)

# Child directories
source(file.path(parent.dir, "Shared Source Code",
                 "Define Child Directories.R"), echo = TRUE)

# Packages
source(file.path(parent.dir, "Shared Source Code", 
                 "Install and Load Packages.R"), echo = TRUE)

# Custom functions
source(file.path(parent.dir, "Shared Source Code", "Define Functions.R"), 
       echo = TRUE)



# Load analysis ####

# Extrapolation Table
extrapolate.rda <- file.path(parent.dir, "Statistical Analysis", 
                             "Extrapolation Table.rda")
load(extrapolate.rda)
head(extrapolate)



# Load Prepared Data ####

# Labor Force Statistics
labor.force.rda <- file.path(parent.dir, "Prepared Data", 
                             "Labor Force Statistics.rda")
load(labor.force.rda)
head(labor_force)

# Standard Deviation of Employment Growth
sd.empl.rda <- file.path(parent.dir, "Prepared Data",
                         "Standard Deviation of Employment Growth.rda")
load(sd.empl.rda)
(sd_diff_empl <- sd_diff_empl[1, sd_diff_empl])

# Standard Deviation of Availability Growth
sd.avail.rda <- file.path(parent.dir, "Prepared Data",
                          "Standard Deviation of Availability Growth.rda")
load(sd.avail.rda)
sd_diff_avail



# Jed Kolko, Broadband and Local Growth ####
# A one standard deviation change in broadband availability corresponds to a 
# 0.085 standard deviation change in employment.

# Lookup
kolko <- copy(extrapolate) # BTOP counties
kolko <- labor_force[kolko] # labor force statistics
setkeyv(kolko, key(sd_diff_avail))
kolko <- sd_diff_avail[kolko] # standard deviation of availability growth

# Calculate effect in standard deviations
kolko[, effect_in_sd := effect / sd_diff_avail]

# Project increase in employment growth rate
kolko[, increased_employment_growth := effect_in_sd * .085 * sd_diff_empl]

# Project increase in employment
kolko[, increased_employment := increased_employment_growth * employed]

# Project economic benefit (increase * average annual wage)
kolko[, economic_benefit := (increased_employment * avg_ann_wage) / 1e6]

# Print totals
kolko[, lapply(.SD, sum), keyby = list(definition, adjustment, match.type), 
      .SDcols = c("increased_employment", "economic_benefit")]




# Gillett, Sharon Eisner, et al., Measuring the Economic Impact of Broadband 
# Deployment ####
# The introduction of any broadband to a ZIP code (from no broadband to at 
# least 1 broadband line) increases the employment growth rate by 1-1.4 percent.
# (.014426)

# Lookup
gillett <- copy(extrapolate) # BTOP counties
gillett <- labor_force[gillett] # labor force statistics

# Coefficient
coef <- exp(log(1.014426) / 4) - 1
# 1.014426 ^ (1/4) - 1

# Project increase in employment
gillett[, increased_employment := effect * coef * employed]

# Project economic benefit
gillett[, economic_benefit := (increased_employment * avg_ann_wage) / 1e6]

# Print totals
gillett[, lapply(.SD, sum), keyby = list(definition, adjustment, match.type), 
        .SDcols = c("increased_employment", "economic_benefit")]

# Four-year totals
d <- gillett[definition == "NTIA" & adjustment == "Availability" & 
               match.type == "Primary"]
d[, em_y1 := effect * coef * employed]
d[, em_y2 := effect * ((1 + coef) ^ 2 - 1) * employed - em_y1]
d[, em_y3 := effect * ((1 + coef) ^ 3 - 1) * employed - em_y1 - em_y2]
d[, em_y4 := effect * ((1 + coef) ^ 4 - 1) * employed - em_y1 - em_y2 - em_y3]
d[, ben_y1 := em_y1 * avg_ann_wage]# / 1e6]
d[, ben_y2 := em_y2 * avg_ann_wage]# / 1e6]
d[, ben_y3 := em_y3 * avg_ann_wage]# / 1e6]
d[, ben_y4 := em_y4 * avg_ann_wage]# / 1e6]
d[, tot_y1 := ben_y1 * 4]
d[, tot_y2 := ben_y2 * 3]
d[, tot_y3 := ben_y3 * 2]
d[, tot_y4 := ben_y4 * 1]
# Total jobs after 4
d[, sum(em_y1 + em_y2 + em_y3 + em_y4)]
# Total ben in year 4
d[, sum(ben_y1 + ben_y2 + ben_y3 + ben_y4)]
# Cumulative ben after 4
d[, sum(tot_y1 + tot_y2 + tot_y3 + tot_y4)]


# Results ####

# Bind together
inc_empl <- Rbind.fill(kolko[, Study := 1],
                       gillett[, Study := 2])
inc_empl[, Study := factor(Study, 1:2, c("Kolko (2010)", 
                                         "Gillett et al. (2006)"))]

# Aggregate tables
(kolko_jobs <- tabular(
  Factor(Study) * Heading("Grants") * (case + Heading("All BTOP") * 1) * 
    Heading("Match Type") * match.type ~ 
    Heading() * adjustment * Heading() * definition * Heading() * 
    increased_employment * Heading() * sum * 
    Format(formatC(digits = 0, format = "f", big.mark = ",")), 
  inc_empl[Study == "Kolko (2010)"][, Study := droplevels(Study)]))

(gillett_jobs <- tabular(
  Factor(Study) * Heading("Grants") * (case + Heading("All BTOP") * 1) * 
    Heading("Match Type") * match.type ~ 
    Heading() * adjustment * Heading() * definition * Heading() *
    increased_employment * Heading() * sum *
    Format(formatC(digits = 0, format = "f", big.mark = ",")), 
  inc_empl[Study == "Gillett et al. (2006)"][, Study := droplevels(Study)]))

(kolko_dollars <- tabular(
  Factor(Study) * Heading("Grants") * (case + Heading("All BTOP") * 1) * 
    Heading("Match Type") * match.type ~ 
    Heading() * adjustment * Heading() * definition * Heading() * 
    economic_benefit * Heading() * sum *
    Format(formatC(digits = 0, format = "f", big.mark = ",")), 
  inc_empl[Study == "Kolko (2010)"][, Study := droplevels(Study)]))

(gillett_dollars <- tabular(
  Factor(Study) * Heading("Grants") * (case + Heading("All BTOP") * 1) * 
    Heading("Match Type") * match.type ~ 
    Heading() * adjustment * Heading() * definition * Heading() *
    economic_benefit * Heading() * sum *
    Format(formatC(digits = 0, format = "f", big.mark = ",")), 
  inc_empl[Study == "Gillett et al. (2006)"][, Study := droplevels(Study)]))

# Write
write.xlsx.tabular(kolko_jobs, file.path(
  appD, 
  "D.9 Long-term Impact of BTOP CCI Infrastructure on Employment - Table 56.xlsx"))

write.xlsx.tabular(gillett_jobs, file.path(
  appD, 
  "D.9 Long-term Impact of BTOP CCI Infrastructure on Employment - Table 57.xlsx"))

write.xlsx.tabular(kolko_dollars, file.path(
  appD,
  "D.9 Long-term Impact of BTOP CCI Infrastructure on Employment - Table 58.xlsx"))

write.xlsx.tabular(gillett_dollars, file.path(
  appD, 
  "D.9 Long-term Impact of BTOP CCI Infrastructure on Employment - Table 59.xlsx"))
