# Prepare workspace ####

# Define parent directory
parent.dir <- "C:/Final Report"

# Reset workspace 
rm(list = setdiff(ls(), "parent.dir"))
gc()

# Strings as characters
options(stringsAsFactors = FALSE)

# Child directories
source(file.path(parent.dir, "Shared Source Code", 
                 "Define Child Directories.R"), echo = TRUE)

# Packages
source(file.path(parent.dir, "Shared Source Code", 
                 "Install and Load Packages.R"), echo = TRUE)

# Custom functions
source(file.path(parent.dir, "Shared Source Code", "Define Functions.R"), 
       echo = TRUE)



# Load analysis ####

# Matching
matching.rda <- file.path(parent.dir, "Statistical Analysis", "Matching.rda")
load(matching.rda)
head(match.data)



# Match results ####

# List different matches
match.list <- c("m.nofa.back.prim", "m.nofa.back.sens",
                "m.nofa.forw.prim", "m.nofa.forw.sens", 
                "m.nofa.unad.prim", "m.nofa.unad.sens", 
                "m.ntia.back.prim", "m.ntia.back.sens", 
                "m.ntia.forw.prim", "m.ntia.forw.sens", 
                "m.ntia.unad.prim", "m.ntia.unad.sens")

# Combine results into a list
matches <- mget(match.list, .GlobalEnv)

# Which availability rates and covariates were used?
avail <- lapply(matches, attr, "availability")
covar <- lapply(matches, attr, "covariates")

# Combine availability rates, population, and covariates used for each
var <- Map("c", avail, "pop_total_2011_06", covar)



# Figure. Number of Different Control Counties Selected for Treatment 
# Counties ####

# Bind matches into one table
matches <- rbindlist(matches)[
  , match := unlist(Map(rep, names(matches), lapply(matches, nrow)))]

# Definition
matches[, Definition := factor(grepl("nofa", match), c(TRUE, FALSE),
                               c("NOFA", "NTIA"))]

# How many different control counties?
matches <- matches[, list(diff_matches = length(unique(control))), 
                   by = list(Definition, treat)]
matches[, diff_matches := factor(diff_matches)]

# Rollup so I can add labels and such
matches <- matches[, .N, by = list(Definition, diff_matches)]

# Figure
(num_diff_controls <- ggplot(matches, aes(diff_matches, N)) + 
   facet_grid(Definition ~ .) + 
   geom_bar(stat = "identity", fill = "#800000") +
   geom_text(aes(label = N), vjust = -.2, size = 6) +
   geom_blank(data = copy(matches)[, N := N * 1.1]) +
   labs(x = "Number of Different Control Counties Selected",
        y = "Number of Treatment Counties") +
   theme_btop())

# Write
ggsave(file.path(appD, "Figure 12.png"), num_diff_controls, height = 6, 
       width = 8)
