# Prepare workspace ####

# Define parent directory
parent.dir <- "C:/Final Report"

# Reset workspace 
rm(list = setdiff(ls(), "parent.dir"))
gc()

# Strings as characters
options(stringsAsFactors = FALSE)

# Child directories
source(file.path(parent.dir, "Shared Source Code",
                 "Define Child Directories.R"), echo = TRUE)

# Packages
source(file.path(parent.dir, "Shared Source Code", 
                 "Install and Load Packages.R"), echo = TRUE)

# Custom functions
source(file.path(parent.dir, "Shared Source Code", "Define Functions.R"),
       echo = TRUE)



# Read NBM data ####

# Data files
nbm.pop.file <- "NBM Statistics, 2011-06-30 and 2013-06-30 - Population.xlsx"
nbm.rural.file <- 
  "NBM Statistics, 2011-06-30 and 2013-06-30 - Rural Population.xlsx"
nbm.minority.file <- 
  "NBM Statistics, 2011-06-30 and 2013-06-30 - Minority Population.xlsx"
nbm.over60.file <- 
  "NBM Statistics, 2011-06-30 and 2013-06-30 - Over 60 Population.xlsx"
nbm.poverty.file <- 
  "NBM Statistics, 2011-06-30 and 2013-06-30 - Poverty Population.xlsx"

# Function to read consistently
Read <- function(file, classes = c(rep("character", 4), rep("numeric", 7))) {
  d <- Read.xlsx2(file.path(parent.dir, "Input Files", file), 1, 
                  colClasses = classes)
  d[, 5:11 := Map(replace, .SD, lapply(.SD, is.nan), NA_real_), .SDcols = 5:11]
  d
}

# Read population file
pop <- Read(nbm.pop.file)
setkey(pop, fips, year, month)

# Replace missing adjustment values with corresponding uandjusted values
pop[year == "2011", `:=`(bbbackward_nofa = bb_nofa, bbbackward_ntia = bb_ntia)]
pop[year == "2013", `:=`(bbforward_nofa = bb_nofa, bbforward_ntia = bb_ntia)]

# Read vulnerable populations files
vuln <- Map(Read, file = c(nbm.rural.file, nbm.minority.file, nbm.over60.file,
                           nbm.poverty.file))



# Reduce vulnerable populations tables ####

# Extract only overall vulnerable population value from each
vuln <- lapply(vuln, function(x) 
  setnames(x[, list(fips, year, month, unit = "pop", total)], "total", 
           x[1, unit]))

# Merge individual tables
vuln <- Reduce(merge, lapply(vuln, setkeyv, c("fips", "year", "month", "unit")))
setnames(vuln, "ruralpop", "rural")



# Percentages ####

# Create table of broadband percentages of population
ppop <- copy(pop)[, unit := "ppop"]
ppop[, 6:11 := lapply(.SD, "/", ppop[, total]), .SDcols = 6:11]
ppop[, total := NULL]

# Create table of vulnerable percentage of population
vuln[pop, total := total]
pvuln <- copy(vuln)[, unit := "ppop"]
pvuln[, 5:8 := lapply(.SD, "/", pvuln[, total]), .SDcols = 5:8]
pvuln[, total := NULL]
vuln[, total := NULL]



# Combine data ####

# Reshape all four tables to long and combine
availability <- rbind.fill(
  melt(pop, id.vars = 1:4), 
  melt(ppop, id.vars = 1:4), 
  melt(vuln, id.vars = 1:4),
  melt(pvuln, id.vars = 1:4))

# Reshape combined long to wide
availability <- Dcast(availability, fips ~ unit + variable + year + month, 
                      value.var = "value")

# Key
setkey(availability, fips)


# Save ####
nbm.rda <- file.path(parent.dir, "Prepared Data", 
                     "NBM Population Statistics.rda")
save(availability, file = nbm.rda)



# Write to file ####
nbm.xlsx <- file.path(parent.dir, "Prepared Data", 
                      "NBM Population Statistics.xlsx")
write.xlsx2(availability, nbm.xlsx, row.names = FALSE)
