# Prepare workspace ####

# Define parent directory
parent.dir <- "C:/Final Report"

# Reset workspace 
rm(list = setdiff(ls(), "parent.dir"))
gc()

# Strings as characters
options(stringsAsFactors = FALSE)

# Child directories
source(file.path(parent.dir, "Shared Source Code", 
                 "Define Child Directories.R"), echo = TRUE)

# Packages
source(file.path(parent.dir, "Shared Source Code",
                 "Install and Load Packages.R"), echo = TRUE)

# Custom functions
source(file.path(parent.dir, "Shared Source Code", "Define Functions.R"),
       echo = TRUE)



# Load Prepared Data ####

# Grants
grants.rda <- file.path(parent.dir, "Prepared Data", "Grants.rda")
load(grants.rda)
head(grants)



# Total users to unique users deflator based on Becker et al. (2010) ####
deflator <- sum(0.23 * 5 * 52, 0.24 * 52, 0.2 * 12, 0.2 * 4, 0.13)



# CAC data - used to determine what percentage of users to include ####

# Read 2011-2 CAC files for PCC site data
cac11.file <- file.path(parent.dir, "Input Files", "CAC Data - 2011.xls")
cac12.file <- file.path(parent.dir, "Input Files", "CAC Data - 2012.xls")
pcc_cac_11 <- Read.xlsx2(cac11.file, sheetName = "PCC Sites Data",
                         colClasses = "character")
pcc_cac_12 <- Read.xlsx2(cac12.file, sheetName = "PCC Sites Data", 
                         colClasses = "character")

# Combine CAC years and rename variables
k0 <- c("Award.ID", "Institution.Type", "Number.of.Users.of.Per.Week")
k1 <- c("Award_Num", "type", "users")
cac_users <- rbind(pcc_cac_11[, k0, with = FALSE], 
                   pcc_cac_12[, k0, with = FALSE])
setnames(cac_users, k0, k1)
cac_users[, users := as.numeric(users)]
cac_users <- cac_users[!is.na(users)]
setkey(cac_users, Award_Num)

# Which PCC location types are applicable?
# View all types in data
cac_users[, sort(unique(type))]
# Applicable types from list in data
applic <- c(
  "Community Based Organization", "Community Based Organization (CBO)", 
  "Library", "Non-Profit Entity", "Non-Profit Organization",
  "Public Housing", "Public Housing ", "Tribal")
# Percentage of users at applicable types
applic_pct <- cac_users[, list(
  applic_pct = sum(replace(users, !type %in% applic, 0) / sum(users))),
  keyby = Award_Num]



# PPR data - used for total number of users ####

# Read users reported in PPRs
ppr.file <- file.path(parent.dir, "Input Files", "PCC Quarterly PPR Data.xlsx")
pam <- Read.xlsx2(ppr.file, sheetIndex = 1)

# Reshape PPR users
quarterly_users <- pam[, list(avg_weekly_users = as.numeric(UserAverage_Num)),
                       keyby = list(
                         Award_Num, year = Year_Desc, 
                         quarter = as.numeric(substr(Quarter_Cd, 2, 2)))]

# There is some duplication...
quarterly_users[quarterly_users[duplicated(quarterly_users),
                                list(Award_Num, year, quarter)]]

# ...but values are identical
unique(setkey(quarterly_users[quarterly_users[
  duplicated(quarterly_users), list(Award_Num, year, quarter)]], NULL))
setkey(quarterly_users, NULL)
quarterly_users <- unique(quarterly_users)
setkey(quarterly_users, Award_Num, year, quarter)



# Combine CAC and PPR data ####

# Lookup
quarterly_users <- applic_pct[quarterly_users]

# Replace any missing percentage with 1 (assumption)
quarterly_users[is.na(applic_pct), applic_pct := 1]

# Calculate the number of average weekly applicable users
quarterly_users[, users_week := avg_weekly_users * applic_pct]

# Subset to just the grante-year-quarters with applicable users
quarterly_users <- quarterly_users[avg_weekly_users > 0]

# Key
setkey(quarterly_users, Award_Num, year, quarter)



# Calculate grants' total unique users ####

# How many users per quarter?
quarterly_users[, users_quarter := users_week * 13]

# Format year and quarter as date
quarterly_users[, begins := as.Date(paste(year, c(1, 4, 7, 10)[quarter], 1), 
                                    "%Y %m %d")]
quarterly_users[, ends := as.Date(paste(year, quarter * 3, 1), "%Y %m %d") + 
                  months(1) - days(1)]

# How many users for all quarters?
# What are the first and last quarters with data?
users <- quarterly_users[, list(begins = min(begins), ends = max(ends), 
                                total_users = sum(users_quarter, na.rm = TRUE)), 
                         keyby = Award_Num]

# How many years for each grant?
users[, years := as.numeric(ends - begins) / 365]

# How many unique users over entire grant period?
users[, unique_users := total_users / (deflator * years)]

# Drop begins, ends, years
users[, c("begins", "ends", "years") := NULL]



# Save ####
users.rda <- file.path(parent.dir, "Prepared Data", "PCC Unique Users.rda")
save(users, file = users.rda)



# Write ####
users.file <- file.path(parent.dir, "Prepared Data", "PCC Unique Users.xlsx")
write.xlsx2(grants[users], users.file, row.names = FALSE)
