# Prepare workspace ####

# Define parent directory
parent.dir <- "C:/Final Report"

# Reset workspace 
rm(list = setdiff(ls(), "parent.dir"))
gc()

# Strings as characters
options(stringsAsFactors = FALSE)

# Child directories
source(file.path(parent.dir, "Shared Source Code", 
                 "Define Child Directories.R"), echo = TRUE)

# Packages
source(file.path(parent.dir, "Shared Source Code", 
                 "Install and Load Packages.R"), echo = TRUE)

# Custom functions
source(file.path(parent.dir, "Shared Source Code", 
                 "Define Functions.R"), echo = TRUE)



# Load Prepared Data ####

# Grants
grants.rda <- file.path(parent.dir, "Prepared Data", "Grants.rda")
load(grants.rda)
head(grants)

# PCC and SBA Service Areas
service.rda <- file.path(parent.dir, "Prepared Data", 
                         "PCC and SBA Service Areas.rda")
load(service.rda)
head(service.areas)
head(county.areas)
head(tract.areas)
head(sa.state.level)



# 2009 ACS data ####

# Read data
acs.file <- file.path(parent.dir, "Input Files", 
                      "Census Bureau ACS (2005-2009) Data.csv")
acs <- fread(acs.file, colClasses = c("character", rep("numeric", 12)))

# Labor force statistics 
acs_lf <- acs[, list(fips, year = 2009, population, population_15_64, 
                     labor_force, employed, unemployed, 
                     unemployment_rate = unemployed / labor_force)]
setkey(acs_lf, fips)



# 2009 LAU data ####

# Read data
lau.file <- file.path(parent.dir, "Input Files",
                      "BLS Local Area Unemployment Statistics - 2009.xlsx")
lau <- Read.xlsx2(lau.file, colClasses = c("character", rep("numeric", 5)), 
                  sheetIndex = 1)
setkey(lau, fips)

# Add on population data
pd <- c("population", "population_15_64")
lau <- acs_lf[, lapply(.SD, sum), keyby = list(fips = substr(fips, 1, 5)), 
              .SDcols = pd][lau]



# 2009 minimum wages ####
minwg.file <- file.path(parent.dir, "Input Files", 
                       "DOL National and State Minimum Wages - 2009.xlsx")
minwg <- Read.xlsx2(minwg.file, sheetIndex = 1, colClasses = c(
  rep("character", 2), "numeric"))
national_minwg <- minwg[state == "National", minimum_wage]
minwg <- minwg[state != "National"][, state := NULL]
minwg[minimum_wage < national_minwg, minimum_wage := national_minwg]
setkey(minwg, state_fips)



# 2009 unemployment duration data ####
dur.file <- file.path(parent.dir, "Input Files", 
                      "BLS-DOL Unemployment Duration - 2009.xlsx")
dur <- Read.xlsx2(dur.file, sheetIndex = 1, colClasses = c(
  "character", rep("numeric", 2)))
setkey(dur, state_fips)



# Get employed, unemployed, etc. ####

# LAU for counties
county_lf <- lau[county.areas]

# ACS for tracts
tract_lf <- acs_lf[tract.areas]

# Combine LAU and ACS results
labor_force <- as.data.table(rbind.fill(county_lf, tract_lf))
setkey(labor_force, fips)



# How much of the labor force in the service area falls in ... ####

# Each county?
lf_county <- labor_force[, list(lf = sum(labor_force)), 
                         keyby = list(fips = substr(fips, 1, 5), Award_Num)]
lf_county[, weight := lf / sum(lf), by = Award_Num]

# Each state?
lf_state <- labor_force[, list(lf = sum(labor_force)), 
                        keyby = list(Award_Num, fips = substr(fips, 1, 2))]
lf_state[, weight := lf / sum(lf), by = Award_Num]



# Get minimum wages ####
# State is lowest available level
setkey(sa.state.level, fips)
wage <- minwg[sa.state.level]
setkey(wage, Award_Num, state_fips)



# Get unemployment duration ####
# State is lowest available level
setkey(sa.state.level, fips)
unemdur <- dur[sa.state.level]
setkey(unemdur, Award_Num, state_fips)



# Aggregate statistics to grant level ####

# Sum people
labor_force <- labor_force[, lapply(.SD, sum), keyby = Award_Num, .SDcols = c(
  "population", "population_15_64", "labor_force", "employed", "unemployed")]

# Weighted average of minimum wage based on percentage of labor force in state
minimum_wage <- wage[lf_state][, list(
  min_wage = weighted.mean(minimum_wage, weight)), keyby = Award_Num]

# Weighted average of unemployment duration based on percentage of labor force 
# in state
unemployment_duration <- unemdur[lf_state][, list(
  avg_duration = weighted.mean(avg_duration, weight)), keyby = Award_Num]

# Merge
labor_force <- unemployment_duration[minimum_wage[labor_force]]



# Calculate rates that we will use ####

# Labor force participation as percentage of population
labor_force[, labor_participation := labor_force / population]

# Labor force participation as percentage of working age population
labor_force[, labor_participation_15_64 := labor_force / population_15_64]

# Unemployment rate
labor_force[, unemployment_rate := unemployed / labor_force]



# Virgin Islands ####
# Only has data for minimum wage
# We will use national rates and averages for others

# National labor force participation and unemployment rates
lau_nat <- lau[!is.na(population), lapply(.SD, sum), .SDcols = c(
  "labor_force", "unemployed", "population", "population_15_64")]
lp_nat <- lau_nat[, labor_force / population]
lp1564_nat <- lau_nat[, labor_force / population_15_64]
un_nat <- lau_nat[, unemployed / labor_force]

# VI minimum wage
min_vi <- minwg[state_fips == "78", minimum_wage]

# National average unemployment duration
lau_st <- lau[, list(lf = sum(labor_force)), 
              keyby = list(state_fips = substr(fips, 1, 2))]
dur_nat <- lau_st[dur][, sum(avg_duration * (lf / sum(lf)))]

# Replace missing Virgin Islands data with above calculations
labor_force[substr(Award_Num, 1, 2) == "78", `:=`
            (labor_participation = lp_nat, 
             labor_participation_15_64 = lp1564_nat, 
             unemployment_rate = un_nat, 
             min_wage = min_vi, 
             avg_duration = dur_nat)]



# Save ####
labor.rda <- file.path(parent.dir, "Prepared Data", 
                       "PCC and SBA Labor Force Statistics.rda")
save(labor_force, file = labor.rda)



# Write ####
labor.xlsx <- file.path(parent.dir, "Prepared Data", 
                        "PCC and SBA Labor Force Statistics.xlsx")
write.xlsx(grants[labor_force], labor.xlsx, row.names = FALSE)
