# Prepare workspace ####

# Define parent directory
parent.dir <- "C:/Final Report"

# Reset workspace 
rm(list = setdiff(ls(), "parent.dir"))
gc()

# Strings as characters
options(stringsAsFactors = FALSE)

# Child directories
source(file.path(parent.dir, "Shared Source Code", 
                 "Define Child Directories.R"), echo = TRUE)

# Packages
source(file.path(parent.dir, "Shared Source Code", 
                 "Install and Load Packages.R"), echo = TRUE)

# Custom functions
source(file.path(parent.dir, "Shared Source Code", "Define Functions.R"), 
       echo = TRUE)



# NBM households data ####

# Read file
nbm.hhu.file <- file.path(
  parent.dir, "Input Files", 
  "NBM Statistics, 2011-06-30 and 2013-06-30 - Housing Units.xlsx")
hhu <- Read.xlsx2(nbm.hhu.file, sheetIndex = 1, 
                  colClasses = c(rep("character", 4), rep("numeric", 7)))

# Only need 2011 NOFA values
hhu <- hhu[year == "2011", list(fips, total, bb_nofa, bbforward_nofa, 
                                bbbackward_nofa)]
setkey(hhu, fips)

# Replace missing adjusted values with corresponding unadjusted
hhu[, bbbackward_nofa := bb_nofa]

# Availability rates
hhu[, 3:5 := lapply(.SD, "/", hhu[, total]), .SDcols = 3:5]
hhu[, total := NULL]



# FCC 477 households data ####

# Read file
fcc477.file <- file.path(
  parent.dir, "Input Files",
  "FCC Form 477 County Household Adoption Levels - 2011-06-30.xlsx")
fcc477 <- Read.xlsx2(fcc477.file, sheetIndex = 1)

# Only need NOFA-level values
fcc477 <- fcc477[
  speed == "At least 768 kbps downstream and greater than 200 kbps upstream", 
  adoption_level, keyby = fips]

# Turn adoption category into number - use midpoint
fcc477[, adopt_min := as.numeric(sapply(strsplit(adoption_level, "-"), "[", 1))]
fcc477[, adopt_max := as.numeric(sapply(strsplit(adoption_level, "-"), "[", 2))]
fcc477[, adoption_level := (adopt_min + adopt_max) / 2]



# Adoption Rates ####

# Merge NBM and FCC 477 data
adoption <- merge(hhu, fcc477, all = TRUE)

# Reshape
adoption <- melt(adoption, id.vars = c("fips", "adoption_level"), 
                 measure.vars = c("bb_nofa", "bbforward_nofa",
                                  "bbbackward_nofa"),
                 value.name = "availability_rate", variable.name = "adjustment")
adoption[, adjustment := factor(adjustment, c(
  "bb_nofa", "bbforward_nofa", "bbbackward_nofa"), c(
    "Availability", "Forward Looking", "Backward Looking"))]

# Calculate adoption rate - cap at one
adoption[, adoption_rate := adoption_level / availability_rate]
adoption[adoption_rate > 1, adoption_rate := 1]

# Finalize table
adoption[, c("adoption_level", "availability_rate") := NULL]
setkey(adoption, fips, adjustment)



# Save ####
adoption.rda <- file.path(
  parent.dir, "Prepared Data", 
  "Rates of Adoption by Households with Availability.rda")
save(adoption, file = adoption.rda)



# Write ####
adoption.xlsx <- file.path(
  parent.dir, "Prepared Data",
  "Rates of Adoption by Households with Availability.xlsx")
write.xlsx2(adoption, adoption.xlsx, row.names = FALSE)
