# Prepare workspace ####

# Define parent directory
parent.dir <- "C:/Final Report"

# Reset workspace 
rm(list = setdiff(ls(), "parent.dir"))
gc()

# Strings as characters
options(stringsAsFactors = FALSE)

# Child directories
source(file.path(parent.dir, "Shared Source Code", 
                 "Define Child Directories.R"), echo = TRUE)

# Packages
source(file.path(parent.dir, "Shared Source Code", 
                 "Install and Load Packages.R"), echo = TRUE)

# Custom functions
source(file.path(parent.dir, "Shared Source Code", "Define Functions.R"), 
       echo = TRUE)



# Load analyzed data ####

# NBM Population Statistics
nbm.rda <- file.path(parent.dir, "Prepared Data", 
                     "NBM Population Statistics.rda")
load(nbm.rda)
head(availability)



# Different service areas ####

# Evaluation study sample
asr.cases.file <- file.path(parent.dir, "Input Files", 
                            "CCI Evaluation Study Sample Service Areas.xlsx")
grant.service.areas <- Read.xlsx2(asr.cases.file, sheetIndex = 1)

# CAC CAI service areas
cac.cai.file <- file.path(parent.dir, "Input Files",
                          "CAC Data - CAI Locations with Counties.xlsx")
cai <- read.xlsx2(cac.cai.file, sheetIndex = 1)$fips

# CAC POP
cac.pop.file <- file.path(parent.dir, "Input Files", 
                          "CAC Data - POP Locations with Counties.xlsx")
pop <- read.xlsx2(cac.pop.file, sheetIndex = 1)
pop <- c(pop$facility.1.fips, pop$facility.2.fips)

# Awarded BTOP grants
asr.btop.file <- file.path(parent.dir, "Input Files", 
                           "Awarded BTOP Service Areas.xlsx")
awarded.btop <- read.xlsx2(asr.btop.file, sheetIndex = 1)$fips

# Awarded BIP grants
asr.bip.file <- file.path(parent.dir, "Input Files", 
                          "Awarded BIP Service Areas.xlsx")
awarded.bip <- read.xlsx2(asr.bip.file, sheetIndex = 1)$fips

# Rejected BTOP grants
asr.rejected.file <- file.path(parent.dir, "Input Files", 
                               "Service Areas of Grant Applications Not Funded by BTOP or BIP.xlsx")
rejected <- Read.xlsx2(asr.rejected.file, sheetIndex = 1)
rejected <- rejected[grepl("BTOP", bip_btop), fips]

# Counties with redacted CAI/POPs
asr.redacted.file <- file.path(
  parent.dir, "Input Files",
  "Awarded BTOP Service Areas with Redacted Locations.xlsx")
redacted <- read.xlsx2(asr.redacted.file, sheetIndex = 1)$fips




# Determine county eligibility for matching ####

# All counties
d <- unique(availability[, list(fips)])

# Evaluation Study Sample counties - treatments
d[, btop_cases := fips %in% grant.service.areas[, fips]]

# CAC location service area - ineligible
d[, btop_cac := fips %in% c(cai, pop)]

# ASR-researched awarded BTOP service areas - ineligible
d[, btop_asr := fips %in% awarded.btop]

# ASR-researched awarded BIP service areas - ineligible
d[, bip_asr := fips %in% awarded.bip]

# Redacted CAC location service area - ineligible
d[, btop_redacted := fips %in% redacted]

# Rejected BTOP grant counties - potential controls
d[, rejected := fips %in% rejected]



# Turn indicators into elimination steps ####

# All counties and equivalents in the United States
d[, all := TRUE]

# Less counties in Alaska, Hawaii, and Outlying Areas
d[, lessakhi := as.numeric(substr(fips, 1, 2)) %in% setdiff(1:56, c(2, 15))]

# Less counties not in proposed BTOP service area
d[, btop_app := lessakhi & (btop_asr | rejected)]

# Less counties in awarded BTOP or BIP grant service area
d[, less_btop_bip := btop_app & !(btop_cases | btop_cac | btop_asr | bip_asr)]



# Determine county match status - treatment, potential control, removed ####
d[(btop_cases),    Group := 1]
d[(less_btop_bip), Group := 2]
d[is.na(Group),    Group := 3]
d[, Group := factor(Group, levels = 1:3, labels = c(
  "Treatment Counties", "Potential Control Counties", "Removed Counties"))]



# Copy to new tables ####
treat.control.status <- d[, Group, keyby = fips]
control.elimination <- d[, list(fips, all, lessakhi, btop_app, less_btop_bip)]
setkey(control.elimination, fips)



# Create table of all BTOP counties ####
btop <- SJ(fips = c(cai, pop, awarded.btop, redacted, 
                    grant.service.areas[, fips]))
btop <- btop[!is.na(fips) & fips != ""]
btop <- unique(btop)
btop[, case := factor(fips %in% grant.service.areas[, fips], c(TRUE, FALSE), 
                      c("Evaluation Study Sample", "Rest of BTOP"))]
setkey(btop, fips)



# Save ####
treat.control.rda <- file.path(parent.dir, "Prepared Data", 
                               "Treatment and Potential Control Counties.rda")
save(treat.control.status, control.elimination, btop, file = treat.control.rda)



# Write ####
treat.control.xlsx <- file.path(parent.dir, "Prepared Data", 
                                "Treatment and Potential Control Counties.xlsx")

write.xlsx2(treat.control.status, treat.control.xlsx, "Matching Status", 
            row.names = FALSE)
write.xlsx2(control.elimination, treat.control.xlsx, "Potential Control Status",
            row.names = FALSE, append = TRUE)
write.xlsx2(btop, treat.control.xlsx, "All BTOP Counties", row.names = FALSE, 
            append = TRUE)
