# Prepare workspace ####

# Define parent directory
parent.dir <- "C:/Final Report"

# Reset workspace 
rm(list = setdiff(ls(), "parent.dir"))
gc()

# Strings as characters
options(stringsAsFactors = FALSE)

# Child directories
source(file.path(parent.dir, "Shared Source Code", 
                 "Define Child Directories.R"), echo = TRUE)

# Packages
source(file.path(parent.dir, "Shared Source Code", 
                 "Install and Load Packages.R"), echo = TRUE)

# Custom functions
source(file.path(parent.dir, "Shared Source Code", "Define Functions.R"), 
       echo = TRUE)




# Load analysis ####

# Matching
matching.rda <- file.path(parent.dir, "Statistical Analysis", "Matching.rda")
load(matching.rda)
head(match.data)



# NBM data ####

# Files
nbm.pop.file <- "NBM Statistics, 2011-06-30 and 2013-06-30 - Population.xlsx"
nbm.rural.file <-
  "NBM Statistics, 2011-06-30 and 2013-06-30 - Rural Population.xlsx"
nbm.minority.file <-
  "NBM Statistics, 2011-06-30 and 2013-06-30 - Minority Population.xlsx"
nbm.over60.file <- 
  "NBM Statistics, 2011-06-30 and 2013-06-30 - Over 60 Population.xlsx"
nbm.poverty.file <- 
  "NBM Statistics, 2011-06-30 and 2013-06-30 - Poverty Population.xlsx"

# Function to read consistently
Read <- function(file, classes = c(rep("character", 4), rep("numeric", 7))) {
  d <- Read.xlsx2(file.path(parent.dir, "Input Files", file), sheetIndex = 1, 
                  colClasses = classes)
  d[, 5:11 := Map(replace, .SD, lapply(.SD, is.nan), NA_real_), .SDcols = 5:11]
  d
}

# Read NBM files
nbm <- lapply(c(nbm.pop.file, nbm.rural.file, nbm.minority.file, 
                nbm.over60.file, nbm.poverty.file), Read)
nbm <- rbindlist(nbm)

# Which are the broadband variables?
bb <- grep("bb", names(nbm), value = TRUE)

# Overall population table, used to calculate non-English
pop <- nbm[unit == "pop"]
pop[, bb := lapply(.SD, "/", pop[, total]), .SDcols = bb, with = FALSE]

# Non-English county-level data merged with overall population
nonen <- match.data[, list(nonenglish = non_english), keyby = fips][pop]

# Calculate non-English broadband as overall availability * non-English population
nonen[, total := total * nonenglish][, unit := "nonenglish"][, nonenglish := NULL]
nonen[, bb := lapply(.SD, "*", nonen[, total]), .SDcols = bb, with = FALSE]

# Combine non-English with rest of data
nbm <- rbind(nbm, nonen)
setkey(nbm, unit, fips, year, month)

# Replace missing adjusted with correct unadjusted
nbm[year == "2011", c("bbbackward_nofa", "bbbbbackward_ntia") := list(bb_nofa, bb_ntia)]
nbm[year == "2013", c("bbforward_nofa", "bbforward_ntia") := list(bb_nofa, bb_ntia)]

# Reshape to long
nbm <- melt(nbm, id.vars = 1:4, measure.vars = c("total", bb))

# Reshape to wide - mimics the match.data table
nbm <- Dcast(nbm, fips ~ unit + variable + year + month)
setkey(nbm, fips)



# Resample to calculate confidence interval around effect ####

# Function to resample effect
incResampleEffect <- function(matches, avail11, avail13, incpops, seed, 
                              data = nbm) {
  
  # Print which set of matches and variables used
  cat(matches, avail11, avail13, "\n")
  
  # Get table of matches
  m <- copy(get(matches))
  
  # Copy data to preserve integrity
  d <- copy(data)
  setkey(d, fips)
  setnames(d, c(avail11, avail13, incpops), 
           c("avail11", "avail13", "p11", "p13"))
  
  # Lookup treatment county values
  setkey(m, treat)
  tr <- d[m][, list(treat = fips, control, avail11, avail13, p11, p13)]
  
  # Lookup control county values
  setkey(m, control)
  co <- d[m][, list(treat, control = fips, avail11, avail13, p11, p13)]
  
  # Merge treatment and control values
  setnames(tr, names(tr)[-1:-2], paste0("tr_", names(tr)[-1:-2]))
  setnames(co, names(co)[-1:-2], paste0("co_", names(co)[-1:-2]))
  tc <- merge(tr, co, by = c("treat", "control")) 
  
  # Function to calculate effect for i row numbers
  didfun <- function(tc, i, ...) {
    x <- tc[i, lapply(.SD, sum), .SDcols = 3:ncol(tc)]
    (x[, tr_avail13 / tr_p13] - x[, tr_avail11 / tr_p11]) - 
      (x[, co_avail13 / co_p13] - x[, co_avail11 / co_p11])
  }
  
  # Resample
  set.seed(seed)
  b <- boot(data = tc, statistic = didfun, R = 1000, sim = "ordinary")
  
  # Get confidence interval
  ci <- boot.ci(b, type = "basic")
  
  # Return table with effect and lower and upper limits
  data.table(effect = b$t0, lower = ci$basic[1, 4], upper = ci$basic[1, 5], 
             matches = matches)
  
}


# Do resampling. Only do the base cases for all the populations. 

# Vulnerable populations
inc <- c("ruralpop", "minority", "over60", "poverty", "nonenglish")

# Match set (base case)
matches <- "m.ntia.unad.prim"

# Vulnerable population availability variables
a11 <- paste0(inc, "_bb_ntia_2011_06")
a13 <- paste0(inc, "_bb_ntia_2013_06")

# Vulnerable population total population variables
tp <- Map(paste0, inc, list(c("_total_2011_06", "_total_2013_06")))

# Get seeds for the different populations
set.seed(314159)
seeds <- sample(10000:99999, length(inc))

# Resample for all populations
incidence <- Map(incResampleEffect, matches, a11, a13, tp, seeds)

# Combine and clean up results
incidence <- rbindlist(incidence)
incidence[, Population := factor(inc, inc, c(
  "Rural", "Minority", "Over 60", "Poverty", "Non-English"))]
incidence[, matches := NULL]



# Save ####
incidence.rda <- file.path(parent.dir, "Statistical Analysis",
                           "Incidence Analysis.rda")
save(incidence, file = incidence.rda)



# Write ####
incidence.xlsx <- file.path(parent.dir, "Statistical Analysis", 
                            "Incidence Analysis.xlsx")
write.xlsx(incidence, incidence.xlsx, row.names = FALSE)
