# Prepare workspace ####

# Define parent directory
parent.dir <- "C:/Final Report"

# Reset workspace 
rm(list = setdiff(ls(), "parent.dir"))
gc()

# Strings as characters
options(stringsAsFactors = FALSE)

# Child directories
source(file.path(parent.dir, "Shared Source Code",
                 "Define Child Directories.R"), echo = TRUE)

# Packages
source(file.path(parent.dir, "Shared Source Code", 
                 "Install and Load Packages.R"), echo = TRUE)

# Custom functions
source(file.path(parent.dir, "Shared Source Code", "Define Functions.R"), 
       echo = TRUE)




# Load Prepared Data ####

# ACS 2010
acs.rda <- file.path(parent.dir, "Prepared Data", "ACS 2010.rda")
load(acs.rda)
head(acs10)

# NBM Population Statistics
nbm.rda <- file.path(parent.dir, "Prepared Data", 
                     "NBM Population Statistics.rda")
load(nbm.rda)
head(availability)

# Treatment and Potential Control Counties
treat.control.rda <- file.path(parent.dir, "Prepared Data", 
                               "Treatment and Potential Control Counties.rda")
load(treat.control.rda)
head(treat.control.status)



# Build matching data set ####

# Merge NBM and ACS data
match.data <- merge(availability, acs10, by = "fips", all = TRUE)

# Log of population
match.data[, lpop_total_2011_06 := log(pop_total_2011_06)]

# Rurality indicator
match.data[, rural := ppop_rural_2011_06 == 1]

# Eligibility for matching
match.data <- match.data[treat.control.status]

# Subset 
cases <- match.data[Group != "Removed Counties"]
setkey(cases, fips)



# Match treatments to controls ####

# Treated vectors
Tr.cases <- cases[, Group == "Treatment Counties"]

# Matching function
matching <- function(availability, covariates, data = cases, Tr = Tr.cases) {
    
  # Covariate matrix
  x <- c(availability, "lpop_total_2011_06", covariates, "rural")
  X <- as.matrix(data[, lapply(.SD, as.numeric), .SDcols = x])
  rownames(X) <- data[, fips]
  
  # Iterate over maximum difference allowed between treatment and control 
  # availability until all treatment are matched
  # Initialize maximum allowed difference
  max.diff <- .01 / 1.1
  # Standard deviation of availability - used to determine caliper
  stdev <- sd(unlist(data[, availability, with = FALSE]))
  # Empty list to store matching results
  matches <- list()
  # Looping condition
  fully.matched <- FALSE
  # Loop until fully matched
  while (!fully.matched) {
    # Maximum allowed difference in availability in percentage points
    max.diff <- max.diff * 1.1
    if (max.diff > 1) break
    # Maximum allowed differences in standard deviations (calipers)
    #   First element is for availability
    #   Second element is for log populatinos
    #   Remaining elements are for everything else - 10 standard deviations 
    #     is high enough that it shouldn't be restrictive
    cal <- c(max.diff / stdev, 0.5, rep(10, ncol(X) - 2))
    # Variables that have to be exact matches
    #   Rural flag (last) element is only variable set to true
    ex <- c(rep(FALSE, 2 + length(covariates)), TRUE)
    # Match
    m <- Match(Tr = Tr, X = X, Weight = 2, caliper = cal, exact = ex)
    # Store the resulting data
    matches[[paste(max.diff)]] <- unique(
      data.table(treat = data[m$index.treated, fips], 
                 control = data[m$index.control, fips],
                 key = "treat"))
    # Test if all were matched
    fully.matched <- m$orig.treated.nobs == m$wnobs
    if (max.diff == 1) break
  }
  
  # Return results
  # Remove matches for BTOP counties that have matches with smaller differences
  matches <- rbindlist(matches)[
    , cal := as.numeric(unlist(Map(rep, names(matches), 
                                   lapply(matches, nrow))))]
  matches[, mincal := cal == min(cal), by = treat]
  matches <- matches[(mincal)][, c("mincal", "cal") := NULL]
  # Key
  setkey(matches, treat)
  # Attributes
  attr(matches, "availability") <- availability
  attr(matches, "covariates") <- covariates
  # Return
  matches
  
}

# Do matching 
# Covariate names (for convenience)
cv <- c("ppop_rural_2011_06", "ppop_over60_2011_06", "ppop_minority_2011_06",
        "ppop_poverty_2011_06", "non_english")
# NTIA definition
#  unadjusted, primary
m.ntia.unad.prim <- matching("ppop_bb_ntia_2011_06", cv[1]) 
#  unadjusted, sensitivity
m.ntia.unad.sens <- matching("ppop_bb_ntia_2011_06", cv) 
#  forward looking, primary
m.ntia.forw.prim <- matching("ppop_bbforward_ntia_2011_06", cv[1]) 
#  forward looking, sensitivity
m.ntia.forw.sens <- matching("ppop_bbforward_ntia_2011_06", cv) 
#  backward looking, primary
m.ntia.back.prim <- matching("ppop_bbbackward_ntia_2011_06", cv[1]) 
#  backward looking, sensitivity
m.ntia.back.sens <- matching("ppop_bbbackward_ntia_2011_06", cv) 
# NOFA definition
#  unadjusted, primary
m.nofa.unad.prim <- matching("ppop_bb_nofa_2011_06", cv[1]) 
#  unadjusted, sensitivity
m.nofa.unad.sens <- matching("ppop_bb_nofa_2011_06", cv) 
#  forward looking, primary
m.nofa.forw.prim <- matching("ppop_bbforward_nofa_2011_06", cv[1]) 
#  forward looking, sensitivity
m.nofa.forw.sens <- matching("ppop_bbforward_nofa_2011_06", cv) 
#  backward looking, primary
m.nofa.back.prim <- matching("ppop_bbbackward_nofa_2011_06", cv[1]) 
#  backward looking, sensitivity
m.nofa.back.sens <- matching("ppop_bbbackward_nofa_2011_06", cv) 



# Save ####
matching.rda <- file.path(parent.dir, "Statistical Analysis", "Matching.rda")
save(m.nofa.unad.prim, m.nofa.unad.sens, m.ntia.unad.prim, m.ntia.unad.sens, 
     m.nofa.forw.prim, m.nofa.forw.sens, m.ntia.forw.prim, m.ntia.forw.sens,
     m.nofa.back.prim, m.nofa.back.sens, m.ntia.back.prim, m.ntia.back.sens,
     match.data, file = matching.rda)


# Write ####
matching.xlsx <- file.path(parent.dir, "Statistical Analysis", "Matching.xlsx")
writeit <- function(data, sheet, append = TRUE) {
  write.xlsx2(x = data, file = matching.xlsx, sheetName = sheet, 
              row.names = FALSE, append = append)
}

writeit(m.ntia.unad.prim, "NTIA Primary", append = FALSE)
writeit(m.nofa.unad.prim, "NOFA Primary")
writeit(m.ntia.unad.sens, "NTIA Sensitivity")
writeit(m.nofa.unad.sens, "NOFA Sensitivity")
writeit(m.ntia.forw.prim, "NTIA Forward Primary")
writeit(m.nofa.forw.prim, "NOFA Forward Primary")
writeit(m.nofa.forw.sens, "NOFA Forward Sensitivity")
writeit(m.ntia.forw.sens, "NTIA Forward Sensitivity")
writeit(m.ntia.back.prim, "NTIA Backward Primary")
writeit(m.nofa.back.prim, "NOFA Backward Primary")
writeit(m.nofa.back.sens, "NOFA Backward Sensitivity")
writeit(m.ntia.back.sens, "NTIA Backward Sensitivity")
writeit(match.data, "Matching Data")
